@@ -46,7 +46,7 @@
 #include <unistd.h>
 
 #include <meta/main.h>
-#include <meta/errors.h>
+#include <meta/meta-x11-errors.h>
 
 #include <gudev/gudev.h>
 
@@ -188,7 +188,7 @@ apply_crtc_assignments (MetaMonitorManager *manager,
               MetaOutput *output = g_ptr_array_index (crtc_info->outputs, j);
 
               output->is_dirty = TRUE;
-              output->crtc = crtc;
+              meta_output_assign_crtc (output, crtc);
             }
         }
 
@@ -249,7 +249,7 @@ apply_crtc_assignments (MetaMonitorManager *manager,
               continue;
             }
 
-          output->crtc = NULL;
+          meta_output_unassign_crtc (output);
           output->is_primary = FALSE;
         }
     }
