@@ -78,44 +78,67 @@
 
 #include "cogl-utils.h"
 #include "clutter-utils.h"
-#include <meta/errors.h>
+#include <meta/meta-x11-errors.h>
 #include "meta-background-actor-private.h"
 #include "meta-background-private.h"
 #include "meta-cullable.h"
+#include "meta/display.h"
 
 enum
 {
-  PROP_META_SCREEN = 1,
+  PROP_META_DISPLAY = 1,
   PROP_MONITOR,
   PROP_BACKGROUND,
+  PROP_GRADIENT,
+  PROP_GRADIENT_HEIGHT,
+  PROP_GRADIENT_MAX_DARKNESS,
   PROP_VIGNETTE,
   PROP_VIGNETTE_SHARPNESS,
-  PROP_BRIGHTNESS
+  PROP_VIGNETTE_BRIGHTNESS
 };
 
 typedef enum {
   CHANGED_BACKGROUND = 1 << 0,
   CHANGED_EFFECTS = 1 << 2,
   CHANGED_VIGNETTE_PARAMETERS = 1 << 3,
+  CHANGED_GRADIENT_PARAMETERS = 1 << 4,
   CHANGED_ALL = 0xFFFF
 } ChangedFlags;
 
-#define VERTEX_SHADER_DECLARATIONS                                      \
+#define GRADIENT_VERTEX_SHADER_DECLARATIONS                             \
+"uniform vec2 scale;\n"                                                 \
+"varying vec2 position;\n"                                              \
+
+#define GRADIENT_VERTEX_SHADER_CODE                                     \
+"position = cogl_tex_coord0_in.xy * scale;\n"                           \
+
+#define GRADIENT_FRAGMENT_SHADER_DECLARATIONS                           \
+"uniform float gradient_height_perc;\n"                                 \
+"uniform float gradient_max_darkness;\n"                                \
+"varying vec2 position;\n"                                              \
+
+#define GRADIENT_FRAGMENT_SHADER_CODE                                                    \
+"float min_brightness = 1.0 - gradient_max_darkness;\n"                                  \
+"float gradient_y_pos = min(position.y, gradient_height_perc) / gradient_height_perc;\n" \
+"float pixel_brightness = (1.0 - min_brightness) * gradient_y_pos + min_brightness;\n"   \
+"cogl_color_out.rgb = cogl_color_out.rgb * pixel_brightness;\n"                          \
+
+#define VIGNETTE_VERTEX_SHADER_DECLARATIONS                             \
 "uniform vec2 scale;\n"                                                 \
 "uniform vec2 offset;\n"                                                \
 "varying vec2 position;\n"                                              \
 
-#define VERTEX_SHADER_CODE                                              \
+#define VIGNETTE_VERTEX_SHADER_CODE                                     \
 "position = cogl_tex_coord0_in.xy * scale + offset;\n"                  \
 
-#define FRAGMENT_SHADER_DECLARATIONS                                    \
+#define VIGNETTE_FRAGMENT_SHADER_DECLARATIONS                           \
 "uniform float vignette_sharpness;\n"                                   \
 "varying vec2 position;\n"                                              \
 
-#define FRAGMENT_SHADER_CODE                                                   \
+#define VIGNETTE_FRAGMENT_SHADER_CODE                                          \
 "float t = 2.0 * length(position);\n"                                          \
 "t = min(t, 1.0);\n"                                                           \
-"float pixel_brightness = 1.0 - t * vignette_sharpness;\n"                       \
+"float pixel_brightness = 1.0 - t * vignette_sharpness;\n"                     \
 "cogl_color_out.rgb = cogl_color_out.rgb * pixel_brightness;\n"                \
 
 typedef struct _MetaBackgroundLayer MetaBackgroundLayer;
@@ -123,17 +146,22 @@ typedef struct _MetaBackgroundLayer MetaBackgroundLayer;
 typedef enum {
   PIPELINE_VIGNETTE = (1 << 0),
   PIPELINE_BLEND = (1 << 1),
+  PIPELINE_GRADIENT = (1 << 2),
 } PipelineFlags;
 
 struct _MetaBackgroundActorPrivate
 {
-  MetaScreen *screen;
+  MetaDisplay *display;
   int monitor;
 
   MetaBackground *background;
 
+  gboolean gradient;
+  double gradient_max_darkness;
+  int gradient_height;
+
   gboolean vignette;
-  double brightness;
+  double vignette_brightness;
   double vignette_sharpness;
 
   ChangedFlags changed;
@@ -186,7 +214,9 @@ get_preferred_size (MetaBackgroundActor *self,
   MetaBackgroundActorPrivate *priv = META_BACKGROUND_ACTOR (self)->priv;
   MetaRectangle monitor_geometry;
 
-  meta_screen_get_monitor_geometry (priv->screen, priv->monitor, &monitor_geometry);
+  meta_display_get_monitor_geometry (priv->display,
+                                     priv->monitor,
+                                     &monitor_geometry);
 
   if (width != NULL)
     *width = monitor_geometry.width;
@@ -231,7 +261,7 @@ meta_background_actor_get_preferred_height (ClutterActor *actor,
 static CoglPipeline *
 make_pipeline (PipelineFlags pipeline_flags)
 {
-  static CoglPipeline *templates[4];
+  static CoglPipeline *templates[8];
   CoglPipeline **templatep;
 
   templatep = &templates[pipeline_flags];
@@ -245,20 +275,42 @@ make_pipeline (PipelineFlags pipeline_flags)
 
       if ((pipeline_flags & PIPELINE_VIGNETTE) != 0)
         {
-          static CoglSnippet *vertex_snippet;
-          static CoglSnippet *fragment_snippet;
+          static CoglSnippet *vignette_vertex_snippet;
+          static CoglSnippet *vignette_fragment_snippet;
 
-          if (!vertex_snippet)
-            vertex_snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_VERTEX,
-                                               VERTEX_SHADER_DECLARATIONS, VERTEX_SHADER_CODE);
+          if (!vignette_vertex_snippet)
+            vignette_vertex_snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_VERTEX,
+                                                        VIGNETTE_VERTEX_SHADER_DECLARATIONS,
+                                                        VIGNETTE_VERTEX_SHADER_CODE);
 
-          cogl_pipeline_add_snippet (*templatep, vertex_snippet);
+          cogl_pipeline_add_snippet (*templatep, vignette_vertex_snippet);
 
-          if (!fragment_snippet)
-            fragment_snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_FRAGMENT,
-                                                 FRAGMENT_SHADER_DECLARATIONS, FRAGMENT_SHADER_CODE);
+          if (!vignette_fragment_snippet)
+            vignette_fragment_snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_FRAGMENT,
+                                                          VIGNETTE_FRAGMENT_SHADER_DECLARATIONS,
+                                                          VIGNETTE_FRAGMENT_SHADER_CODE);
 
-          cogl_pipeline_add_snippet (*templatep, fragment_snippet);
+          cogl_pipeline_add_snippet (*templatep, vignette_fragment_snippet);
+        }
+
+      if ((pipeline_flags & PIPELINE_GRADIENT) != 0)
+        {
+          static CoglSnippet *gradient_vertex_snippet;
+          static CoglSnippet *gradient_fragment_snippet;
+
+          if (!gradient_vertex_snippet)
+            gradient_vertex_snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_VERTEX,
+                                                        GRADIENT_VERTEX_SHADER_DECLARATIONS,
+                                                        GRADIENT_VERTEX_SHADER_CODE);
+
+          cogl_pipeline_add_snippet (*templatep, gradient_vertex_snippet);
+
+          if (!gradient_fragment_snippet)
+            gradient_fragment_snippet = cogl_snippet_new (COGL_SNIPPET_HOOK_FRAGMENT,
+                                                          GRADIENT_FRAGMENT_SHADER_DECLARATIONS,
+                                                          GRADIENT_FRAGMENT_SHADER_CODE);
+
+          cogl_pipeline_add_snippet (*templatep, gradient_fragment_snippet);
         }
 
       if ((pipeline_flags & PIPELINE_BLEND) == 0)
@@ -283,6 +335,8 @@ setup_pipeline (MetaBackgroundActor   *self,
     pipeline_flags |= PIPELINE_BLEND;
   if (priv->vignette && clutter_feature_available (CLUTTER_FEATURE_SHADERS_GLSL))
     pipeline_flags |= PIPELINE_VIGNETTE;
+  if (priv->gradient && clutter_feature_available (CLUTTER_FEATURE_SHADERS_GLSL))
+    pipeline_flags |= PIPELINE_GRADIENT;
 
   if (priv->pipeline &&
       pipeline_flags != priv->pipeline_flags)
@@ -325,9 +379,29 @@ setup_pipeline (MetaBackgroundActor   *self,
       priv->changed &= ~CHANGED_VIGNETTE_PARAMETERS;
     }
 
+  if ((priv->changed & CHANGED_GRADIENT_PARAMETERS) != 0)
+    {
+      MetaRectangle monitor_geometry;
+      float gradient_height_perc;
+
+      meta_display_get_monitor_geometry (priv->display,
+                                         priv->monitor, &monitor_geometry);
+      gradient_height_perc = MAX (0.0001, priv->gradient_height / (float)monitor_geometry.height);
+      cogl_pipeline_set_uniform_1f (priv->pipeline,
+                                    cogl_pipeline_get_uniform_location (priv->pipeline,
+                                                                        "gradient_height_perc"),
+                                    gradient_height_perc);
+      cogl_pipeline_set_uniform_1f (priv->pipeline,
+                                    cogl_pipeline_get_uniform_location (priv->pipeline,
+                                                                        "gradient_max_darkness"),
+                                    priv->gradient_max_darkness);
+
+      priv->changed &= ~CHANGED_GRADIENT_PARAMETERS;
+    }
+
   if (priv->vignette)
     {
-      color_component = priv->brightness * opacity / 255.;
+      color_component = priv->vignette_brightness * opacity / 255.;
 
       if (!clutter_feature_available (CLUTTER_FEATURE_SHADERS_GLSL))
         {
@@ -478,28 +552,46 @@ meta_background_actor_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_META_SCREEN:
-      priv->screen = g_value_get_object (value);
+    case PROP_META_DISPLAY:
+      priv->display = g_value_get_object (value);
       break;
     case PROP_MONITOR:
-      priv->monitor = g_value_get_int (value);
+      meta_background_actor_set_monitor (self, g_value_get_int (value));
       break;
     case PROP_BACKGROUND:
       meta_background_actor_set_background (self, g_value_get_object (value));
       break;
+    case PROP_GRADIENT:
+      meta_background_actor_set_gradient (self,
+                                          g_value_get_boolean (value),
+                                          priv->gradient_height,
+                                          priv->gradient_max_darkness);
+      break;
+    case PROP_GRADIENT_HEIGHT:
+      meta_background_actor_set_gradient (self,
+                                          priv->gradient,
+                                          g_value_get_int (value),
+                                          priv->gradient_max_darkness);
+      break;
+    case PROP_GRADIENT_MAX_DARKNESS:
+      meta_background_actor_set_gradient (self,
+                                          priv->gradient,
+                                          priv->gradient_height,
+                                          g_value_get_double (value));
+      break;
     case PROP_VIGNETTE:
       meta_background_actor_set_vignette (self,
                                           g_value_get_boolean (value),
-                                          priv->brightness,
+                                          priv->vignette_brightness,
                                           priv->vignette_sharpness);
       break;
     case PROP_VIGNETTE_SHARPNESS:
       meta_background_actor_set_vignette (self,
                                           priv->vignette,
-                                          priv->brightness,
+                                          priv->vignette_brightness,
                                           g_value_get_double (value));
       break;
-    case PROP_BRIGHTNESS:
+    case PROP_VIGNETTE_BRIGHTNESS:
       meta_background_actor_set_vignette (self,
                                           priv->vignette,
                                           g_value_get_double (value),
@@ -521,8 +613,8 @@ meta_background_actor_get_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_META_SCREEN:
-      g_value_set_object (value, priv->screen);
+    case PROP_META_DISPLAY:
+      g_value_set_object (value, priv->display);
       break;
     case PROP_MONITOR:
       g_value_set_int (value, priv->monitor);
@@ -530,11 +622,20 @@ meta_background_actor_get_property (GObject      *object,
     case PROP_BACKGROUND:
       g_value_set_object (value, priv->background);
       break;
+    case PROP_GRADIENT:
+      g_value_set_boolean (value, priv->gradient);
+      break;
+    case PROP_GRADIENT_HEIGHT:
+      g_value_set_int (value, priv->gradient_height);
+      break;
+    case PROP_GRADIENT_MAX_DARKNESS:
+      g_value_set_double (value, priv->gradient_max_darkness);
+      break;
     case PROP_VIGNETTE:
       g_value_set_boolean (value, priv->vignette);
       break;
-    case PROP_BRIGHTNESS:
-      g_value_set_double (value, priv->brightness);
+    case PROP_VIGNETTE_BRIGHTNESS:
+      g_value_set_double (value, priv->vignette_brightness);
       break;
     case PROP_VIGNETTE_SHARPNESS:
       g_value_set_double (value, priv->vignette_sharpness);
@@ -563,14 +664,14 @@ meta_background_actor_class_init (MetaBackgroundActorClass *klass)
   actor_class->get_paint_volume = meta_background_actor_get_paint_volume;
   actor_class->paint = meta_background_actor_paint;
 
-  param_spec = g_param_spec_object ("meta-screen",
-                                    "MetaScreen",
-                                    "MetaScreen",
-                                    META_TYPE_SCREEN,
+  param_spec = g_param_spec_object ("meta-display",
+                                    "MetaDisplay",
+                                    "MetaDisplay",
+                                    META_TYPE_DISPLAY,
                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
 
   g_object_class_install_property (object_class,
-                                   PROP_META_SCREEN,
+                                   PROP_META_DISPLAY,
                                    param_spec);
 
   param_spec = g_param_spec_int ("monitor",
@@ -593,6 +694,36 @@ meta_background_actor_class_init (MetaBackgroundActorClass *klass)
                                    PROP_BACKGROUND,
                                    param_spec);
 
+  param_spec = g_param_spec_boolean ("gradient",
+                                     "Gradient",
+                                     "Whether gradient effect is enabled",
+                                     FALSE,
+                                     G_PARAM_READWRITE);
+
+  g_object_class_install_property (object_class,
+                                   PROP_GRADIENT,
+                                   param_spec);
+
+  param_spec = g_param_spec_int ("gradient-height",
+                                 "Gradient Height",
+                                 "Height of gradient effect",
+                                 0, G_MAXINT, 0,
+                                 G_PARAM_READWRITE);
+
+  g_object_class_install_property (object_class,
+                                   PROP_GRADIENT_HEIGHT,
+                                   param_spec);
+
+  param_spec = g_param_spec_double ("gradient-max-darkness",
+                                    "Gradient Max Darkness",
+                                    "How dark is the gradient initially",
+                                     0.0, 1.0, 0.0,
+                                     G_PARAM_READWRITE);
+
+  g_object_class_install_property (object_class,
+                                   PROP_GRADIENT_MAX_DARKNESS,
+                                   param_spec);
+
   param_spec = g_param_spec_boolean ("vignette",
                                      "Vignette",
                                      "Whether vignette effect is enabled",
@@ -604,13 +735,13 @@ meta_background_actor_class_init (MetaBackgroundActorClass *klass)
                                    param_spec);
 
   param_spec = g_param_spec_double ("brightness",
-                                    "Brightness",
+                                    "Vignette Brightness",
                                     "Brightness of vignette effect",
                                     0.0, 1.0, 1.0,
                                     G_PARAM_READWRITE);
 
   g_object_class_install_property (object_class,
-                                   PROP_BRIGHTNESS,
+                                   PROP_VIGNETTE_BRIGHTNESS,
                                    param_spec);
 
   param_spec = g_param_spec_double ("vignette-sharpness",
@@ -633,8 +764,12 @@ meta_background_actor_init (MetaBackgroundActor *self)
                                                    META_TYPE_BACKGROUND_ACTOR,
                                                    MetaBackgroundActorPrivate);
 
+  priv->gradient = FALSE;
+  priv->gradient_height = 0;
+  priv->gradient_max_darkness = 0.0;
+
   priv->vignette = FALSE;
-  priv->brightness = 1.0;
+  priv->vignette_brightness = 1.0;
   priv->vignette_sharpness = 0.0;
 }
 
@@ -647,13 +782,13 @@ meta_background_actor_init (MetaBackgroundActor *self)
  * Return value: the newly created background actor
  */
 ClutterActor *
-meta_background_actor_new (MetaScreen *screen,
-                           int         monitor)
+meta_background_actor_new (MetaDisplay *display,
+                           int          monitor)
 {
   MetaBackgroundActor *self;
 
   self = g_object_new (META_TYPE_BACKGROUND_ACTOR,
-                       "meta-screen", screen,
+                       "meta-display", display,
                        "monitor", monitor,
                        NULL);
 
@@ -749,6 +884,62 @@ meta_background_actor_set_background (MetaBackgroundActor *self,
   clutter_actor_queue_redraw (CLUTTER_ACTOR (self));
 }
 
+void
+meta_background_actor_set_gradient (MetaBackgroundActor *self,
+                                    gboolean             enabled,
+                                    int                  height,
+                                    double               max_darkness)
+{
+  MetaBackgroundActorPrivate *priv;
+  gboolean changed = FALSE;
+
+  g_return_if_fail (META_IS_BACKGROUND_ACTOR (self));
+  g_return_if_fail (height >= 0);
+  g_return_if_fail (max_darkness >= 0. && max_darkness <= 1.);
+
+  priv = self->priv;
+
+  enabled = enabled != FALSE && height != 0;
+
+  if (enabled != priv->gradient)
+    {
+      priv->gradient = enabled;
+      invalidate_pipeline (self, CHANGED_EFFECTS);
+      changed = TRUE;
+    }
+
+  if (height != priv->gradient_height || max_darkness != priv->gradient_max_darkness)
+    {
+      priv->gradient_height = height;
+      priv->gradient_max_darkness = max_darkness;
+      invalidate_pipeline (self, CHANGED_GRADIENT_PARAMETERS);
+      changed = TRUE;
+    }
+
+  if (changed)
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (self));
+}
+
+void
+meta_background_actor_set_monitor (MetaBackgroundActor *self,
+                                   int                  monitor)
+{
+  MetaBackgroundActorPrivate *priv = self->priv;
+  MetaRectangle old_monitor_geometry;
+  MetaRectangle new_monitor_geometry;
+  MetaDisplay *display = priv->display;
+
+  if(priv->monitor == monitor)
+      return;
+
+  meta_display_get_monitor_geometry (display, priv->monitor, &old_monitor_geometry);
+  meta_display_get_monitor_geometry (display, monitor, &new_monitor_geometry);
+  if(old_monitor_geometry.height != new_monitor_geometry.height)
+      invalidate_pipeline (self, CHANGED_GRADIENT_PARAMETERS);
+
+  priv->monitor = monitor;
+}
+
 void
 meta_background_actor_set_vignette (MetaBackgroundActor *self,
                                     gboolean             enabled,
@@ -773,9 +964,9 @@ meta_background_actor_set_vignette (MetaBackgroundActor *self,
       changed = TRUE;
     }
 
-  if (brightness != priv->brightness || sharpness != priv->vignette_sharpness)
+  if (brightness != priv->vignette_brightness || sharpness != priv->vignette_sharpness)
     {
-      priv->brightness = brightness;
+      priv->vignette_brightness = brightness;
       priv->vignette_sharpness = sharpness;
       invalidate_pipeline (self, CHANGED_VIGNETTE_PARAMETERS);
       changed = TRUE;
