@@ -43,21 +43,16 @@
 #include "ui.h"
 #include "util-private.h"
 #include "display-private.h"
+#include "x11/meta-x11-display-private.h"
 
 static gboolean restart_helper_started = FALSE;
 static gboolean restart_message_shown = FALSE;
 static gboolean is_restart = FALSE;
 
 void
-meta_restart_init (void)
+meta_set_is_restart (gboolean whether)
 {
-  Display *xdisplay = meta_ui_get_display ();
-  Atom atom_restart_helper = XInternAtom (xdisplay, "_MUTTER_RESTART_HELPER", False);
-  Window restart_helper_window = None;
-
-  restart_helper_window = XGetSelectionOwner (xdisplay, atom_restart_helper);
-  if (restart_helper_window)
-    is_restart = TRUE;
+  is_restart = whether;
 }
 
 static void
@@ -103,7 +98,7 @@ restart_message_painted (gpointer data)
 
 /**
  * meta_restart:
- * @message: message to display to the user.
+ * @message: (allow-none): message to display to the user, or %NULL
  *
  * Starts the process of restarting the compositor. Note that Mutter's
  * involvement here is to make the restart visually smooth for the
@@ -126,7 +121,7 @@ meta_restart (const char *message)
     MUTTER_LIBEXECDIR "/mutter-restart-helper", NULL
   };
 
-  if (meta_display_show_restart_message (display, message))
+  if (message && meta_display_show_restart_message (display, message))
     {
       /* Wait until the stage was painted */
       clutter_threads_add_repaint_func_full (CLUTTER_REPAINT_FLAGS_POST_PAINT,
@@ -186,17 +181,6 @@ meta_restart (const char *message)
   return;
 }
 
-void
-meta_restart_finish (void)
-{
-  if (is_restart)
-    {
-      Display *xdisplay = meta_display_get_xdisplay (meta_get_display ());
-      Atom atom_restart_helper = XInternAtom (xdisplay, "_MUTTER_RESTART_HELPER", False);
-      XSetSelectionOwner (xdisplay, atom_restart_helper, None, CurrentTime);
-    }
-}
-
 /**
  * meta_is_restart:
  *
