@@ -34,11 +34,12 @@
 #include <X11/Xatom.h>
 #include <X11/extensions/Xfixes.h>
 
-#include <meta/errors.h>
+#include <meta/meta-x11-errors.h>
 #include "meta-xwayland.h"
 #include "meta-xwayland-private.h"
 #include "meta-xwayland-selection-private.h"
 #include "meta-wayland-data-device.h"
+#include "x11/meta-x11-display-private.h"
 
 #define INCR_CHUNK_SIZE (128 * 1024)
 #define XDND_VERSION 5
@@ -466,7 +467,7 @@ x11_selection_data_send_finished (MetaSelectionBridge *selection,
                          gdk_x11_get_xatom_by_name ("DELETE"),
                          gdk_x11_get_xatom_by_name ("_META_SELECTION"),
                          selection->window,
-                         CurrentTime);
+                         META_CURRENT_TIME);
     }
 
   xdnd_send_finished (selection->x11_selection->selection_data,
@@ -562,7 +563,7 @@ wayland_selection_data_new (XSelectionRequestEvent *request_event,
                             MetaWaylandCompositor  *compositor)
 {
   MetaDisplay *display = meta_get_display ();
-  MetaScreen *screen = display->screen;
+  MetaX11Display *x11_display = display->x11_display;
   MetaWaylandDataDevice *data_device;
   MetaWaylandDataSource *wayland_source;
   MetaSelectionBridge *selection;
@@ -609,11 +610,11 @@ wayland_selection_data_new (XSelectionRequestEvent *request_event,
   data->cancellable = g_cancellable_new ();
   data->stream = g_unix_input_stream_new (p[0], TRUE);
 
-  data->window = meta_display_lookup_x_window (meta_get_display (),
-                                               data->request_event.requestor);
+  data->window = meta_x11_display_lookup_x_window (x11_display,
+                                                   data->request_event.requestor);
 
   /* Do *not* change the event mask on the root window, bugger! */
-  if (!data->window && data->request_event.requestor != screen->xroot)
+  if (!data->window && data->request_event.requestor != x11_display->xroot)
     {
       /* Not a managed window, set the PropertyChangeMask
        * for INCR deletion notifications.
@@ -648,15 +649,15 @@ static void
 wayland_selection_data_free (WaylandSelectionData *data)
 {
   MetaDisplay *display = meta_get_display ();
-  MetaScreen *screen = display->screen;
+  MetaX11Display *x11_display = display->x11_display;
 
   /* Do *not* change the event mask on the root window, bugger! */
-  if (!data->window && data->request_event.requestor != screen->xroot)
+  if (!data->window && data->request_event.requestor != x11_display->xroot)
     {
-      meta_error_trap_push (display);
+      meta_x11_error_trap_push (x11_display);
       XSelectInput (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
                     data->request_event.requestor, NoEventMask);
-      meta_error_trap_pop (display);
+      meta_x11_error_trap_pop (x11_display);
     }
 
   g_cancellable_cancel (data->cancellable);
@@ -1343,7 +1344,7 @@ pick_drop_surface (MetaWaylandCompositor *compositor,
   ClutterPoint pos;
 
   clutter_event_get_coords (event, &pos.x, &pos.y);
-  focus_window = meta_stack_get_default_focus_window_at_point (display->screen->stack,
+  focus_window = meta_stack_get_default_focus_window_at_point (display->stack,
                                                                NULL, NULL,
                                                                pos.x, pos.y);
   return focus_window ? focus_window->surface : NULL;
@@ -1615,7 +1616,7 @@ meta_xwayland_selection_handle_xfixes_selection_notify (MetaWaylandCompositor *c
                              gdk_x11_get_xatom_by_name ("TARGETS"),
                              gdk_x11_get_xatom_by_name ("_META_SELECTION"),
                              selection->window,
-                             CurrentTime);
+                             META_CURRENT_TIME);
           XFlush (xdisplay);
         }
     }
@@ -1630,7 +1631,7 @@ meta_xwayland_selection_handle_xfixes_selection_notify (MetaWaylandCompositor *c
       if (event->owner != None && event->owner != selection->window &&
           focus && meta_xwayland_is_xwayland_surface (focus))
         {
-          selection->client_message_timestamp = CurrentTime;
+          selection->client_message_timestamp = META_CURRENT_TIME;
           selection->source = meta_wayland_data_source_xwayland_new (selection);
           meta_wayland_data_device_set_dnd_source (&compositor->seat->data_device,
                                                    selection->source);
@@ -1672,9 +1673,9 @@ meta_xwayland_selection_handle_event (XEvent *xevent)
       return meta_xwayland_selection_handle_client_message (compositor, xevent);
     default:
       {
-        MetaDisplay *display = meta_get_display ();
+        MetaX11Display *x11_display = meta_get_display ()->x11_display;
 
-        if (xevent->type - display->xfixes_event_base == XFixesSelectionNotify)
+        if (xevent->type - x11_display->xfixes_event_base == XFixesSelectionNotify)
           return meta_xwayland_selection_handle_xfixes_selection_notify (compositor, xevent);
 
         return FALSE;
@@ -1701,7 +1702,7 @@ meta_selection_bridge_ownership_notify (struct wl_listener *listener,
       XSetSelectionOwner (xdisplay,
                           selection->selection_atom,
                           selection->window,
-                          CurrentTime);
+                          META_CURRENT_TIME);
     }
 }
 
